#this script tests and fits measurement models for the trait information
#it uses a file containing the intake battery for the N=1939 users
#fits the models on N=1,380 users who did not forecast
#then uses the parameters from that model to obtain trait scores for the
#N=559 users who did forecast
#it also tests whether the factor scores are better predictors than raw scale scores



rm(list=ls())
setwd("~/S1") # Set Working Directory Path

library(reshape2)
library(plyr)
library(dplyr)
library(stringr)
library(ggplot2)
library(standardize)
library(car)
library(mirt)
library(tidyr)
library(psych)
library(papaja)


#intake data including demographic information, psychometric raw scores and scored responses to individual items. 
#Dataset includes N = 1,939 registered users, many of whom dropped out before participating in forecasting
#Original raw responses were not available. 
hfc <- read.csv("season 1 user info.csv")
rcta <- read.csv("season 1 forecasts.csv")

#subset non-active participants
rcta$active <- TRUE
rcta <- unique(rcta[c("user_id", "active")])
hfc <- left_join(hfc,rcta[c("user_id","active")])

hfc_sub <- hfc[is.na(hfc$active),]
hfc_core <- hfc[!is.na(hfc$active),]



names(hfc_sub)
#The following code broke the full intake data into separate data frames for each scale, for simplicity
mtxs <- hfc_sub[paste0("mtx", 1:11)]
aots <- (hfc_sub[paste0("aot", 1:8)])
nfcs <- (hfc_sub[paste0("nfc", 1:18)])
nss <- (hfc_sub[paste0("ns", 1:9)])
bns <- (hfc_sub[paste0("bn", 1:4)])
pks <- (hfc_sub[paste0("pk", 2:50)])
crts <- (hfc_sub[paste0("crt", 1:6)])

mtxs_cr <- hfc_core[paste0("mtx", 1:11)]
aots_cr <- (hfc_core[paste0("aot", 1:8)])
nfcs_cr <- (hfc_core[paste0("nfc", 1:18)])
nss_cr <- (hfc_core[paste0("ns", 1:9)])
bns_cr <- (hfc_core[paste0("bn", 1:4)])
pks_cr <- (hfc_core[paste0("pk", 2:50)])
crts_cr <- (hfc_core[paste0("crt", 1:6)])

#combined scales for PCA
full_scrs <- hfc_sub[c("crtScore","bnScore","nsScore","mtxScore","aotScore","nfcScore",
                       "pkScore")]


#pca
raw.pca <- prcomp(full_scrs,center=T,scale=T)

para <- fa.parallel(full_scrs, fm = "ml",
                    n.iter = 30 * ncol(full_scrs), plot = FALSE)



#plot component and FA solutions
plotdat <- data.frame(Eigenvalues_fa = c(para$fa.values,
                                         para$fa.sim),
                      Eigenvalues_pc = c(para$pc.values,
                                         para$pc.sim),
                      Components = c(1:length(para$fa.values),
                                     1:length(para$fa.sim)),
                      Source = c(rep("Data", length(para$fa.values)),
                                 rep("Simulated", length(para$fa.sim))))


png("pc_para.png", width = 8, height = 4, units = "in", res = 300)

ggplot(plotdat, aes(x = Components, y = Eigenvalues_pc,
                    group = Source, colour = Source, linetype = Source)) + geom_line() + geom_point() + 
  ylab("Eigenvalues") + xlab("Components")  + 
  scale_x_continuous(breaks = 1:ncol(full_scrs)) +
  theme_apa() +
  theme(legend.text=element_text(size=12))

dev.off()

write.csv(raw.pca$rotation, "floadings.csv")

summary(raw.pca)

#bifactor model

#combine intelligence measures

int_mes <- cbind(crts,bns,nss,mtxs)

#.8, .7, .73, .56, .64, .85, .44
#fit bifactor model
int_mod <- mirt.model("CRT = 1-6
                       BN = 7-10
                       NS = 11-19
                       MTX = 20-30
                       G = 1-30")


mirt_int <- mirt(int_mes,int_mod,itemtype = "2PL",method="QMCEM")

facts_int <- summary(mirt_int, rotate="oblimin")$rotF

facts_int <- data.frame(facts_int)

facts_int$scale <- c(rep("crt",6),rep("bn",4),rep("ns",9),rep("mtx",11))

ag1 <- aggregate(G  ~ scale, facts_int, mean)

int_m2 <- M2(mirt_int,QMC=T)


#obtain intelligence factor scores from core sample
#using results obtained from non-participants
int_coefs <- data.frame(coef(mirt_int, simplify = T)$items)

sv_int <- mirt(int_mes,int_mod,itemtype = "2PL",method="QMCEM",
               pars = 'values')


names(int_coefs)

sv_int$value[sv_int$name == 'a1'] <-int_coefs$a1[!is.na(int_coefs$a1)]
sv_int$value[sv_int$name == 'a2'] <-int_coefs$a2[!is.na(int_coefs$a2)]
sv_int$value[sv_int$name == 'a3'] <-int_coefs$a3[!is.na(int_coefs$a3)]
sv_int$value[sv_int$name == 'a4'] <-int_coefs$a4[!is.na(int_coefs$a4)]
sv_int$value[sv_int$name == 'a5'] <-int_coefs$a5[!is.na(int_coefs$a5)]
sv_int$value[sv_int$name == 'd'] <- int_coefs$d[!is.na(int_coefs$d)]
sv_int$value[sv_int$name == 'g'] <- int_coefs$g[!is.na(int_coefs$g)]
sv_int$value[sv_int$name == 'u'] <-int_coefs$u[!is.na(int_coefs$u)]

int_mes_cr <- cbind(crts_cr,bns_cr,nss_cr,mtxs_cr)

mrt_out <- mirt(int_mes_cr, model = int_mod, pars=sv_int, itemtype = "2PL",
                method="QMCEM")


summary(mrt_out)
int_out <- data.frame(fscores(mrt_out, method="EAP",QMC=T))


hfc_core <- data.frame(cbind(hfc_core,g_int=int_out$G))




#combine intelligence measures

cog_mes <- cbind(aots,nfcs)

#fit bifactor model
cog_mod <- mirt.model("AOT = 1-8
                       NFC = 9-26
                       C = 1-26")


mirt_cog <- mirt(cog_mes,cog_mod,itemtype = "graded",method="QMCEM",
                 na.rm=T)

facts_cog <- summary(mirt_cog, rotate="oblimin")$rotF

facts_cog <- data.frame(facts_cog)

facts_cog$scale <- c(rep("aot",8),rep("nfc",18))

ag2 <- aggregate(C  ~ scale, facts_cog, mean)

cog_m2 <- M2(mirt_cog,QMC=T,na.rm=T)


#obtain cognitive style factor scores from core sample
#using parameters obtained from non-participants
cog_coefs <- data.frame(coef(mirt_cog, simplify = T)$items)

sv_cog <- mirt(cog_mes,cog_mod,itemtype = "graded",method="QMCEM",
               pars = 'values')


names(cog_coefs)

sv_cog$value[sv_cog$name == 'a1'] <-cog_coefs$a1[!is.na(cog_coefs$a1)]
sv_cog$value[sv_cog$name == 'a2'] <-cog_coefs$a2[!is.na(cog_coefs$a2)]
sv_cog$value[sv_cog$name == 'a3'] <-cog_coefs$a3[!is.na(cog_coefs$a3)]
sv_cog$value[sv_cog$name == 'd1'] <- cog_coefs$d1[!is.na(cog_coefs$d1)]
sv_cog$value[sv_cog$name == 'd2'] <- cog_coefs$d2[!is.na(cog_coefs$d2)]
sv_cog$value[sv_cog$name == 'd3'] <-cog_coefs$d3[!is.na(cog_coefs$d3)]

cog_mes_cr <- cbind(aots_cr,nfcs_cr)

mrt_out <- mirt(cog_mes_cr, model = cog_mod, pars=sv_cog, itemtype = "graded",
                method="QMCEM")


cog_out <- data.frame(fscores(mrt_out, method="EAP",QMC=T))



hfc_core <- data.frame(cbind(hfc_core,cog_style=cog_out$C))

write.csv(ag1, "G Factor Loadings.csv")
write.csv(ag2, "C Factor Loadings.csv")


write.csv(hfc_core, "s1 core user data.csv", row.names=F)
