rm(list=ls())
setwd("~/S2") # Set Working Directory Path
options(mc.cores = parallel::detectCores()) 

library(tidyverse)
library(zoo)
library(rstan)
rstan_options(auto_write = TRUE)


## Set global seed
set.seed(201305)

## Draw 13 random seed values for models
seeds <- round(runif(13)*100000, 0)

## Load data
ifps <- read.csv("season 2 questions.csv")
br_ifps <- read.csv("season 2 forecasts.csv")

## Merge partition information
br_ifps <- left_join(br_ifps, ifps[c("discover_id","e_l0","e_l1","e_l2",
                                       "e_l3","e_l4","e_l5","e_l6","e_l7",
                                       "e_l8","e_l9","e_l10","e_l_f")])

## Convert Brier to accuracy (eq 6)
br_ifps$correct <- 1-(sqrt(br_ifps$brier/2))

## Convert accuracy to normalized accuracy via probit link
br_ifps$correct_prob <- ifelse(br_ifps$correct == 1, .999,
                               ifelse(br_ifps$correct == 0, .001,
                                      br_ifps$correct))


br_ifps$correct_probit <- VGAM::probitlink(br_ifps$correct_prob)


## Compute and scale time remaining fro each forecast
br_ifps$time_rem <- as.numeric(as.Date(br_ifps$end_date) - as.Date(br_ifps$date))



br_ifps$time_rem <- ifelse(br_ifps$time_rem < 0, 0, 
                           br_ifps$time_rem)


br_ifps$time_rem_scl <- br_ifps$time_rem / 365

## Stan is interfaced via RStan in R using the stan() function
## The first argument is the .Stan file that specifies the model
## the next argument is the data, which must be entered as a list
## The list assigns numeric variables which align with 
## The data block in the .Stan file.
## In this file
## I = the number of forecasters
## J = is the number of Questions
## N = the number of total observations
## ii = the index of each forecaster at each observation (1 to I)
## jj = the index of each forecaster at each observation (1 to J)
## time_rem = time remaining (note that it is negated)
## y = normalized accuracy of the forecast

## Save as new file to compute IRT accuracy across all items
br_ifps_all <- br_ifps

## Convert user and question IDs to numeric for model (repeated separately for all models)
br_ifps_all$caster <- as.numeric(as.factor(br_ifps_all$user_id))
br_ifps_all$item <- as.numeric(as.factor(br_ifps_all$discover_id))


post_n_all <- stan("IRT accuracy.stan",
                   data = list(I = length(unique(br_ifps_all$caster)),
                               J = length(unique(br_ifps_all$item)),
                               N = nrow(br_ifps_all),
                               ii = br_ifps_all$caster,
                               jj = br_ifps_all$item,
                               time_rem = -br_ifps_all$time_rem_scl,
                               y = br_ifps_all$correct_probit),
                   iter = 3000, warmup = 1000, seed = seeds[1], 
                   control = list(adapt_delta = .92, max_treedepth = 15))

write.csv(summary(post_n_all)$summary, "post_n_all.csv")





## Calibration Sample
br_ifps_e_f <- br_ifps[br_ifps$e_l_f == "e",]


br_ifps_e_f$caster <- as.numeric(as.factor(br_ifps_e_f$user_id))
br_ifps_e_f$item <- as.numeric(as.factor(br_ifps_e_f$discover_id))


post_n_e_f <- stan("IRT accuracy.stan",
                   data = list(I = length(unique(br_ifps_e_f$caster)),
                               J = length(unique(br_ifps_e_f$item)),
                               N = nrow(br_ifps_e_f),
                               ii = br_ifps_e_f$caster,
                               jj = br_ifps_e_f$item,
                               time_rem = -br_ifps_e_f$time_rem_scl,
                               y = br_ifps_e_f$correct_probit),
                   iter = 3000, warmup = 1000, seed = seeds[1], 
                   control = list(adapt_delta = .92, max_treedepth = 15))

write.csv(summary(post_n_e_f)$summary, "post_n_e_f.csv")


## Training sample
br_ifps_l_f <- br_ifps[br_ifps$e_l_f == "l",]


br_ifps_l_f$caster <- as.numeric(as.factor(br_ifps_l_f$user_id))
br_ifps_l_f$item <- as.numeric(as.factor(br_ifps_l_f$discover_id))

s1 <- Sys.time()

post_n2_l_f <- stan("IRT accuracy.stan",
                   data = list(I = length(unique(br_ifps_l_f$caster)),
                               J = length(unique(br_ifps_l_f$item)),
                               N = nrow(br_ifps_l_f),
                               ii = br_ifps_l_f$caster,
                               jj = br_ifps_l_f$item,
                               time_rem = -br_ifps_l_f$time_rem_scl,
                               y = br_ifps_l_f$correct_probit),
                   iter = 4000, warmup = 1000, seed = seeds[2], 
                   control = list(adapt_delta = .95, max_treedepth = 15))

write.csv(summary(post_n2_l_f)$summary, "post_n_l_f.csv")


## Partitions of 200, 180...20
br_ifps_e10 <- br_ifps[br_ifps$e_l10 == "e",]


br_ifps_e10$caster <- as.numeric(as.factor(br_ifps_e10$user_id))
br_ifps_e10$item <- as.numeric(as.factor(br_ifps_e10$discover_id))

s1a <- Sys.time()

post_n_e10 <- stan("IRT accuracy.stan",
                             data = list(I = length(unique(br_ifps_e10$caster)),
                                         J = length(unique(br_ifps_e10$item)),
                                         N = nrow(br_ifps_e10),
                                         ii = br_ifps_e10$caster,
                                         jj = br_ifps_e10$item,
                                         time_rem = -br_ifps_e10$time_rem_scl,
                                         y = br_ifps_e10$correct_probit),
                             iter = 4000, warmup = 1000, seed = seeds[3], 
                             control = list(adapt_delta = .92, max_treedepth = 15))

write.csv(summary(post_n_e10)$summary, "post_n_e10.csv")

s2a <- Sys.time()

br_ifps_l10 <- br_ifps[br_ifps$e_l10 == "l",]

br_ifps_l10$caster <- as.numeric(as.factor(br_ifps_l10$user_id))
br_ifps_l10$item <- as.numeric(as.factor(br_ifps_l10$discover_id))


post_n_l10 <- stan("IRT accuracy.stan",
                             data = list(I = length(unique(br_ifps_l10$caster)),
                                         J = length(unique(br_ifps_l10$item)),
                                         N = nrow(br_ifps_l10),
                                         ii = br_ifps_l10$caster,
                                         jj = br_ifps_l10$item,
                                         time_rem = -br_ifps_l10$time_rem_scl,
                                         y = br_ifps_l10$correct_probit),
                             iter = 3000, warmup = 1000, seed = seeds[4], 
                             control = list(adapt_delta = .92, max_treedepth = 15))

write.csv(summary(post_n_l10)$summary, "post_n_l10.csv")


br_ifps_e9 <- br_ifps[br_ifps$e_l9 == "e" & !is.na(br_ifps$e_l9),]


br_ifps_e9$caster <- as.numeric(as.factor(br_ifps_e9$user_id))
br_ifps_e9$item <- as.numeric(as.factor(br_ifps_e9$discover_id))

s1 <- Sys.time()

post_n_e9 <- stan("IRT accuracy.stan",
                 data = list(I = length(unique(br_ifps_e9$caster)),
                             J = length(unique(br_ifps_e9$item)),
                             N = nrow(br_ifps_e9),
                             ii = br_ifps_e9$caster,
                             jj = br_ifps_e9$item,
                             time_rem = -br_ifps_e9$time_rem_scl,
                             y = br_ifps_e9$correct_probit),
                 iter = 3000, warmup = 1000, seed = seeds[5], 
                 control = list(adapt_delta = .92, max_treedepth = 15))

write.csv(summary(post_n_e9)$summary, "post_n_e9.csv")

s2 <- Sys.time()

br_ifps_e8 <- br_ifps[br_ifps$e_l8 == "e" & !is.na(br_ifps$e_l8),]


br_ifps_e8$caster <- as.numeric(as.factor(br_ifps_e8$user_id))
br_ifps_e8$item <- as.numeric(as.factor(br_ifps_e8$discover_id))

s1 <- Sys.time()

post_n_e8 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e8$caster)),
                            J = length(unique(br_ifps_e8$item)),
                            N = nrow(br_ifps_e8),
                            ii = br_ifps_e8$caster,
                            jj = br_ifps_e8$item,
                            time_rem = -br_ifps_e8$time_rem_scl,
                            y = br_ifps_e8$correct_probit),
                iter = 3000, warmup = 1000, seed = seeds[6], 
                control = list(adapt_delta = .92, max_treedepth = 15))

write.csv(summary(post_n_e8)$summary, "post_n_e8.csv")

s2 <- Sys.time()


br_ifps_e7 <- br_ifps[br_ifps$e_l7 == "e" & !is.na(br_ifps$e_l7),]


br_ifps_e7$caster <- as.numeric(as.factor(br_ifps_e7$user_id))
br_ifps_e7$item <- as.numeric(as.factor(br_ifps_e7$discover_id))

s1 <- Sys.time()

post_n_d_e7 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e7$caster)),
                            J = length(unique(br_ifps_e7$item)),
                            N = nrow(br_ifps_e7),
                            ii = br_ifps_e7$caster,
                            jj = br_ifps_e7$item,
                            time_rem = -br_ifps_e7$time_rem_scl,
                            y = br_ifps_e7$correct_probit),
                iter = 3000, warmup = 1000, seed = seeds[7], 
                control = list(adapt_delta = .95, max_treedepth = 15))

# Divergences
write.csv(summary(post_n_d_e7)$summary, "post_n_e7.csv")

s2 <- Sys.time()

br_ifps_e6 <- br_ifps[br_ifps$e_l6 == "e" & !is.na(br_ifps$e_l6),]


br_ifps_e6$caster <- as.numeric(as.factor(br_ifps_e6$user_id))
br_ifps_e6$item <- as.numeric(as.factor(br_ifps_e6$discover_id))

s1 <- Sys.time()

post_n_b_e6 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e6$caster)),
                            J = length(unique(br_ifps_e6$item)),
                            N = nrow(br_ifps_e6),
                            ii = br_ifps_e6$caster,
                            jj = br_ifps_e6$item,
                            time_rem = -br_ifps_e6$time_rem_scl,
                            y = br_ifps_e6$correct_probit),
                iter = 3000, warmup = 1000, seed = seeds[8], 
                control = list(adapt_delta = .92, max_treedepth = 15))
#bulkess
write.csv(summary(post_n_b_e6)$summary, "post_n_e6.csv")

s2 <- Sys.time()


br_ifps_e5 <- br_ifps[br_ifps$e_l5 == "e" & !is.na(br_ifps$e_l5),]


br_ifps_e5$caster <- as.numeric(as.factor(br_ifps_e5$user_id))
br_ifps_e5$item <- as.numeric(as.factor(br_ifps_e5$discover_id))

s1 <- Sys.time()

post_n_b_e5 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e5$caster)),
                            J = length(unique(br_ifps_e5$item)),
                            N = nrow(br_ifps_e5),
                            ii = br_ifps_e5$caster,
                            jj = br_ifps_e5$item,
                            time_rem = -br_ifps_e5$time_rem_scl,
                            y = br_ifps_e5$correct_probit),
                iter = 5000, warmup = 1000, seed = seeds[9], 
                control = list(adapt_delta = .92, max_treedepth = 15))
#bulkess
write.csv(summary(post_n_b_e5)$summary, "post_n_e5.csv")

s2 <- Sys.time()



br_ifps_e4 <- br_ifps[br_ifps$e_l4 == "e" & !is.na(br_ifps$e_l4),]


br_ifps_e4$caster <- as.numeric(as.factor(br_ifps_e4$user_id))
br_ifps_e4$item <- as.numeric(as.factor(br_ifps_e4$discover_id))

s1 <- Sys.time()

post_n_b_e4 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e4$caster)),
                            J = length(unique(br_ifps_e4$item)),
                            N = nrow(br_ifps_e4),
                            ii = br_ifps_e4$caster,
                            jj = br_ifps_e4$item,
                            time_rem = -br_ifps_e4$time_rem_scl,
                            y = br_ifps_e4$correct_probit),
                iter = 5000, warmup = 1000, seed = seeds[10], 
                control = list(adapt_delta = .92, max_treedepth = 15))
#bulkess
write.csv(summary(post_n_b_e4)$summary, "post_n_e4.csv")


s2 <- Sys.time()



br_ifps_e3 <- br_ifps[br_ifps$e_l3 == "e" & !is.na(br_ifps$e_l3),]


br_ifps_e3$caster <- as.numeric(as.factor(br_ifps_e3$user_id))
br_ifps_e3$item <- as.numeric(as.factor(br_ifps_e3$discover_id))

s1 <- Sys.time()

post_n2_b4_e3 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e3$caster)),
                            J = length(unique(br_ifps_e3$item)),
                            N = nrow(br_ifps_e3),
                            ii = br_ifps_e3$caster,
                            jj = br_ifps_e3$item,
                            time_rem = -br_ifps_e3$time_rem_scl,
                            y = br_ifps_e3$correct_probit),
                iter = 5000, warmup = 1000, seed = seeds[11], 
                control = list(adapt_delta = .95, max_treedepth = 15))

write.csv(summary(post_n2_b4_e3)$summary, "post_n2_e3.csv")

s2 <- Sys.time()



br_ifps_e2 <- br_ifps[br_ifps$e_l2 == "e" & !is.na(br_ifps$e_l2),]


br_ifps_e2$caster <- as.numeric(as.factor(br_ifps_e2$user_id))
br_ifps_e2$item <- as.numeric(as.factor(br_ifps_e2$discover_id))

s1 <- Sys.time()

post_n2_b5_e2 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e2$caster)),
                            J = length(unique(br_ifps_e2$item)),
                            N = nrow(br_ifps_e2),
                            ii = br_ifps_e2$caster,
                            jj = br_ifps_e2$item,
                            time_rem = -br_ifps_e2$time_rem_scl,
                            y = br_ifps_e2$correct_probit),
                iter = 5000, warmup = 1000, seed = seeds[12], 
                control = list(adapt_delta = .95, max_treedepth = 15))

write.csv(summary(post_n2_b5_e2)$summary, "post_n2_e2.csv")

s2 <- Sys.time()




br_ifps_e1 <- br_ifps[br_ifps$e_l1 == "e" & !is.na(br_ifps$e_l1),]


br_ifps_e1$caster <- as.numeric(as.factor(br_ifps_e1$user_id))
br_ifps_e1$item <- as.numeric(as.factor(br_ifps_e1$discover_id))



post_n2_b3_e1 <- stan("IRT accuracy.stan",
                data = list(I = length(unique(br_ifps_e1$caster)),
                            J = length(unique(br_ifps_e1$item)),
                            N = nrow(br_ifps_e1),
                            ii = br_ifps_e1$caster,
                            jj = br_ifps_e1$item,
                            time_rem = -br_ifps_e1$time_rem_scl,
                            y = br_ifps_e1$correct_probit),
                iter = 3000, warmup = 1000, seed = seeds[13], 
                control = list(adapt_delta = .95, max_treedepth = 15))


write.csv(summary(post_n2_b3_e1)$summary, "post_e1.csv")

save.image("warningcheck3.RData")






